@@ -19,12 +19,14 @@ check.args.cplm <- function(call,n.obs){
 }
 
 check.args.bcplm <- function(call, n.beta, n.chains){
+  n.iter <- eval(call$n.iter)
+  n.burnin <- eval(call$n.burnin)
   # check counts related inputs
   if (!is.null(call$n.chains) && (!is.numeric(call$n.chains) 
       || call$n.chains < 1))
     stop("'n.chains' must be greater than 1" )
-  if (!is.null(call$n.burnin) && !is.null(call$n.iter) && 
-      call$n.burnin >= call$n.iter)
+  if (!is.null(n.burnin) && !is.null(n.iter) && 
+    n.burnin >= n.iter)
   	stop("'n.burnin' should be less than 'n.iter'" )
   if (!is.null(call$prior.beta.mean) && 
   	length(call$prior.beta.mean) != n.beta)
@@ -40,12 +42,9 @@ check.args.bcplm <- function(call, n.beta, n.chains){
 
 # check initial values in cpglm
 check.inits.cpglm <- function(inits, n.beta){
-  if (!("beta" %in% names(inits)))
-    stop("the 'beta' component in 'inits' is missing") 
-  if (!("phi" %in% names(inits)))
-    stop("the 'phi' component in 'inits' is missing")
-  if (!("p" %in% names(inits)))
-    stop("the 'p' component in 'inits' is missing")
+  
+  if (any(is.na(match(c("beta", "phi", "p"), names(inits)))))
+    stop("'inits' must contain 'beta', 'phi' and 'p'!")
   if (length(inits$beta) != n.beta)
     stop(gettextf("number of 'beta' in 'inits' is %d, but should 
                     equal %d (number of mean parameters)", 
@@ -59,51 +58,37 @@ check.inits.cpglm <- function(inits, n.beta){
 
 # check initial values in cpglmm
 check.inits.cpglmm <- function(inits, n.beta, n.term){
+  check.inits.cpglm(inits, n.beta)
   if (!("Sigma" %in% names(inits)))
     stop("the 'Sigma' component in 'inits' is missing") 
   if (length(inits$Sigma) != n.term) 
     stop(gettextf("'Sigma' in 'inits' should be of length %d", n.term))
 }
 
-# check initial values in bcpglm
-check.inits.bcpglm <- function(inits, n.beta, n.chains){
+# check initial values in bcplm
+check.inits.bcplm <- function(inits, n.beta, n.term, n.chains){
+  
+  if (any(is.na(match(c("beta", "phi", "p", "u", "Sigma"), names(inits)))))
   if (length(inits) != n.chains)
     stop(gettextf("'inits' should be of length %d", n.chains))
-  invisible(lapply(inits, check.inits.cpglm, n.beta))
-}
-
-# check initial values in bcpglmm
-check.inits.bcpglmm <- function(inits, n.beta, n.term, n.chains){
-  if (length(inits) != n.chains)
-    stop(gettextf("'inits' should be of length %d", n.chains))
-  invisible(lapply(inits, function(x) check.inits.cpglmm(x, n.beta, n.term)))
+  lapply(inits, function(x) {
+    check.inits.cpglmm(x, n.beta, n.term)
+    if (!("u" %in% names(x)))
+      stop("the 'u' component in 'inits' is missing")
+    })
 }  
 
 
 ###########################################################
 # default control options   
 ###########################################################
-# control options intializer
-cpglm.control <- function(bound.p = c(1.01,1.99),
-                        trace = 0,
-                        max.iter = 200L){ 
-  if (min(bound.p)<1 || max(bound.p)>2)
-        stop("value of 'bound.p' must be between 1 and 2")        
-  if (!is.numeric(trace) && !is.logical(trace))
-        stop("'trace' must be logical or numeric")
-  if (!is.numeric(max.iter) || max.iter < 0)
-        stop("'max.iter' must be greater than 0")
-  list(bound.p = sort(bound.p),         
-        trace = as.integer(trace),
-        max.iter = as.integer(max.iter))  
-}
 
-
-# update control parameters in cpglmm 
-cpglmm.control <- function(max.iter = 300L,
-                       max.fun = 20000L,               
+# set control parameters  
+cplm.control <- function(max.iter = 300L,
+                       max.fun = 2000L,               
                        bound.p = c(1.01, 1.99),
-                       trace = 0){         
+                       trace = 0,
+                       PQL.init = TRUE){         
   if (!is.numeric(max.iter) || max.iter <= 0) 
         stop("value of 'max.iter' must be > 0")
   if (!is.numeric(max.fun) || max.fun <= 0) 
@@ -118,32 +103,10 @@ cpglmm.control <- function(max.iter = 300L,
   list(max.iter = as.integer(max.iter),
        max.fun = as.integer(max.fun),
        bound.p = as.numeric(sort(bound.p)),
-       trace = as.integer(trace))  
+       trace = as.integer(trace),
+       PQL.init = as.logical(PQL.init))
 }
 
-###########################################################
-# parse and default prior info for the variance component
-###########################################################
-
-# inverse gamma
-igamma <- function(scale = 0.01, shape = 0.01){
-  return(list(igamma.scale = scale, igamma.shape = shape))
-}  
-
-# inverse wishart
-iwish <- function(df = 3, scale = diag(1, df)){
-  return(list(iwish.df = df, iwish.scale = scale))
-}  
-
-# default prior info for the variance component
-prior.Sigma.default <- function(Sigma){
-  lapply(Sigma, function(x){
-        nc <- ncol(x)
-        if (nc == 1) 
-          igamma(scale = 0.001, shape = 0.001) else
-          iwish(df = as.double(nc))
-  })
-}
 
 ###########################################################
 # numerical derivatives  
@@ -160,6 +123,7 @@ grad <- function(parm, fun, ...){
     parm[i] <- parm[i] + 2 * eps
     g2 <- fun(parm, ...)
     gd[i] <- (g2 - g1) / (2 * eps)
+    parm[i] <- parm[i] - eps
   }
   return(gd)
 }
@@ -175,6 +139,7 @@ hess <- function(parm, fun, ...){
     parm[i] <- parm[i] + 2 * eps
     g2 <- grad(parm, fun, ...)
     hn[i,] <- (g2 - g1) / ( 2 * eps)
+    parm[i] <- parm[i] - eps
   }
   return(hn)  
 }
@@ -185,13 +150,13 @@ hess <- function(parm, fun, ...){
 ###########################################################
 
 # construct model frame in cpglm   
-cpglm.mf <- function(mf, contrast){  
+cpglm.mf <- function(mf, contrasts){  
     m <- match(c("formula", "data", "subset", "weights",
                  "na.action", "offset"), names(mf), 0L)
     mf <- mf[c(1L, m)]
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
-    mf <- eval(mf, parent.frame())
+    mf <- eval(mf, parent.frame(2))
     mt <- attr(mf, "terms")
     Y <- model.response(mf, "any")
     X <- if (!is.empty.model(mt)) 
@@ -209,12 +174,45 @@ cpglm.mf <- function(mf, contrast){
 
 # fit a Tweedie glm given a model frame 
 cpglm.fit <- function(fr, p = 1.5, link.power = 0) {
-  fm <- tweedie(var.power = p,link.power = link.power)
+  fm <- tweedie(var.power = p, link.power = link.power)
   int <- attr(attr(fr$mf,"terms"), "intercept") > 0L
-  glm.fit(fr$X, fr$Y, weights = fr$wts, offset = fr$off,
-          family = fm, intercept = int)
+  suppressWarnings(glm.fit(fr$X, fr$Y, weights = fr$wts, offset = fr$off,
+                      family = fm, intercept = int))
 }
   
+# generate inital values for a Tweedie glm given a model frame
+cpglm.init <- function(fr, link.power = 0){
+  p <- 1.5
+  fit <- cpglm.fit(fr, p, link.power)
+  beta <- as.numeric(fit$coefficients)
+  phi <- sum(fit$weights * fit$residuals^2) / fit$df.residual
+  vbeta <- stats:::vcov.glm(fit)
+  list(beta = beta, phi = phi, p = p, vcov = vbeta)
+}
+
+# generate inital values for bcplm
+bcplm.init <- function(fr, link.power = 0, n.chains, bound.p, dm){
+  init <- cpglm.init(fr, link.power)
+  n.beta <- length(init$beta)
+  bound.p[1] <- max(bound.p[1], 1.4)
+  bound.p[2] <- min(bound.p[2], 1.6)
+  init0 <- unname(unlist(init[1:3]))
+  inits <- vector("list", n.chains)
+  inits[[1]] <- init0
+  if (n.chains > 1){
+    for (i in 2:n.chains)
+    inits[[i]] <- c(as.numeric(init$beta + rnorm(n.beta, 0, 0.5)),
+                    runif(1, 0.8 * init$phi, 1.2 * init$phi),
+                    runif(1, bound.p[1], bound.p[2]))
+  }
+  if (!is.null(dm)){
+    s <- lapply(dm$ST, function(x) x %*% t(x))
+    sv <- unlist(lapply(s, as.numeric))
+    inits <- lapply(inits, function(x) c(x, rnorm(dm$dd[["q"]]), sv))
+  }  
+  return(list(inits = inits, vbeta = init$vcov))
+}             
+  
 # compute fitted values of for bigglm  
 fitted.bigglm <- function(object, data, ...){
   # get chunks of data
@@ -262,18 +260,11 @@ fitted.bigglm <- function(object, data, ...){
 # general utility functions  
 ###########################################################
 
-# function to compute log density 
-dtweedie.nlogl <- function(y, mu, phi,power) {
-    ans <- -2 * sum(log(dtweedie(y = y, mu = mu, phi = phi, power = power)))
-    if (is.infinite(ans)) {
-        ans <- sum(tweedie.dev(y = y, mu = mu, power = power)) / length(y)
-    }    
-    #attr(ans, "gradient") <- dtweedie.dldphi(y = y, mu = mu, 
-    #    phi = phi, power = power)
-    ans
+# function to compute minus twice log density 
+dtweedie.nlogl <- function(y, mu, phi, p) {
+    -2 * sum(log(dtweedie(y = y, mu = mu, phi = phi, power = p)))
 }
-    
-  
+
 # function to take inverse of a matrix using svd 
 svd.inv <- function(x){
   sx <- svd(x)
@@ -293,3 +284,44 @@ make.link.power <- function(link) {
     link  
 }
 
+# optimize an objective function using different optimizers
+cplm_optim <- function(par, fn, gr = NULL, ..., 
+                       lower = -Inf, upper = Inf, control = cplm.control(), 
+                       optimizer = "nlminb"){
+  optimizer <- match.arg(optimizer, c("nlminb", "L-BFGS-B", "bobyqa"))
+  if (optimizer == "nlminb"){
+    ans <- nlminb(par, fn, gradient = gr, ..., 
+                  lower = lower, upper = upper,                
+                  control = list(trace = control$trace,
+                                 iter.max = control$max.iter,
+                                 eval.max = control$max.fun))
+    names(ans)[2] <- "value"
+    return(ans[c("par", "value", "convergence", "message")])    
+  } else if (optimizer == "L-BFGS-B"){
+    ans <- optim(par, fn, gr = gr, ..., method = "L-BFGS-B",                     
+                 lower = lower, upper = upper,
+                 control = list(trace = control$trace,
+                                maxit = control$max.iter))
+    return(ans[c("par", "value", "convergence", "message")])
+  } else if (optimizer == "bobyqa"){ 
+    ans <- bobyqa(par, fn, lower = lower, upper = upper, 
+                  control = list(iprint = control$trace,
+                                 rhobe = 0.02, rhoend = 2e-7,
+                                 maxfun = control$max.fun),
+                  ...)
+    names(ans)[c(2, 4, 5)] <- c("value", "convergence", "message")
+    return(ans[c("par", "value", "convergence", "message")])    
+  }
+}
+
+
+# 1-d random walk metropolis
+metrop_rw <- function(n = 1, par = 0, sd = 1, fun, ..., 
+                      lower = -Inf, upper = Inf){
+  fn <- function(x) fun(x, ...)
+  if (par < lower || par > upper)
+    stop("starting value of x is invalid")
+  # run the metropolis algorithm
+  .Call("bcplm_metrop_rw", as.integer(n), as.double(par), as.double(sd), 
+        as.double(lower), as.double(upper), fn, new.env())
+}
